package net.faintedge.radial.examples;

import net.engio.mbassy.listener.Handler;
import net.faintedge.radial.Context;
import net.faintedge.radial.Grid2D;
import net.faintedge.radial.RadialApplication;
import net.faintedge.radial.components.Position;
import net.faintedge.radial.components.RenderSprite;
import net.faintedge.radial.input.events.KeyHeldEvent;

import com.artemis.Entity;
import com.artemis.World;
import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Input;
import com.badlogic.gdx.backends.lwjgl.LwjglApplication;
import com.badlogic.gdx.backends.lwjgl.LwjglApplicationConfiguration;

public class SimpleInput extends RadialApplication {

	private Grid2D grid;
	private Position position;

	@Handler
	public void handleKeyHeld(KeyHeldEvent ev) {
		final float movespeed = 0.5f;
		if (ev.getKeycode() == Input.Keys.W) {
			position.addY(movespeed * ev.getDuration());
		} else if (ev.getKeycode() == Input.Keys.S) {
			position.addY(-movespeed * ev.getDuration());
		} else if (ev.getKeycode() == Input.Keys.A) {
			position.addX(-movespeed * ev.getDuration());
		} else if (ev.getKeycode() == Input.Keys.D) {
			position.addX(movespeed * ev.getDuration());
		}
	}

	@Override
	public void create() {
		super.create();
		
		Context context = super.getContext();
		World world = context.getWorld();
		
		final Entity e = world.createEntity();
		e.addComponent(position = new Position(0, 0));
		e.addComponent(new RenderSprite().fromTexture(Gdx.files.internal("data/libgdx.png"), 0, 0, 512, 275, 0.5f, 0.5f * (275.0f/512.0f)).toRender());
		e.addToWorld();

		context.getEventBus().subscribe(this);
		
		grid = new Grid2D(context);
	}
	
	@Override
	public void render() {
		super.render();
		grid.render();
	}
	
	public static void main(String[] args) {
		LwjglApplicationConfiguration cfg = new LwjglApplicationConfiguration();
		cfg.title = "SimpleInput";
		cfg.width = 800;
		cfg.height = 600;
		
		new LwjglApplication(new SimpleInput(), cfg);
	}
	
}
