package net.faintedge.radial;

import net.engio.mbassy.bus.BusConfiguration;
import net.engio.mbassy.bus.MBassador;
import net.faintedge.radial.input.InputManager;
import net.faintedge.radial.systems.CameraSystem;
import net.faintedge.radial.systems.RenderingSystem;

import com.artemis.World;
import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.Camera;
import com.badlogic.gdx.graphics.OrthographicCamera;
import com.badlogic.gdx.graphics.g2d.SpriteBatch;
import com.badlogic.gdx.graphics.glutils.ShapeRenderer;
import com.google.common.eventbus.EventBus;

public class Context {
	
	private InputManager inputManager;

	private Camera defaultCamera;
	private Camera currentCamera;
	private SpriteBatch batch;
	private World world;
	private ShapeRenderer shapeRenderer;
	
	private RenderingSystem renderingSystem;
	
    private MBassador<Object> eventBus = new MBassador<Object>(new BusConfiguration());
	
	public void create() {
		float w = Gdx.graphics.getWidth();
		float h = Gdx.graphics.getHeight();
		
		inputManager = new InputManager(eventBus);

		defaultCamera = new OrthographicCamera(1, h / w);
		currentCamera = defaultCamera;
		
		batch = new SpriteBatch();
		shapeRenderer = new ShapeRenderer();

		world = new World();
		
		world.setSystem(new CameraSystem());
		
		renderingSystem = new RenderingSystem();
		renderingSystem.setSpriteBatch(batch);
		world.setSystem(renderingSystem, true);

		world.initialize();
	}
	
	public void activate() {
		Gdx.input.setInputProcessor(inputManager);
	}
	
	public void dispose() {
		batch.dispose();
		ResourceManager.getInstance().disposeAll();
	}

	public void setCurrentCamera(Camera currentCamera) {
		this.currentCamera = currentCamera;
	}

	public Camera getCurrentCamera() {
		return currentCamera;
	}

	public SpriteBatch getBatch() {
		return batch;
	}

	public World getWorld() {
		return world;
	}

	public RenderingSystem getRenderingSystem() {
		return renderingSystem;
	}

	public ShapeRenderer getShapeRenderer() {
		return shapeRenderer;
	}

	public InputManager getInputManager() {
		return inputManager;
	}

	public Camera getDefaultCamera() {
		return defaultCamera;
	}

	public MBassador<Object> getEventBus() {
		return eventBus;
	}
	
	
}
