package net.faintedge.radial;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.GL10;
import com.badlogic.gdx.graphics.OrthographicCamera;
import com.badlogic.gdx.graphics.glutils.ShapeRenderer;
import com.badlogic.gdx.graphics.glutils.ShapeRenderer.ShapeType;

public class Grid2D {
	
	private Context context;
	
	public Grid2D(Context context) {
		this.context = context;
	}
	
	public void render() {
		assert context.getCurrentCamera() instanceof OrthographicCamera;
		OrthographicCamera camera = (OrthographicCamera) context.getCurrentCamera();
		
		ShapeRenderer shapeRenderer = context.getShapeRenderer();
		
		float xMin = camera.position.x - camera.viewportWidth/2;
		float xMax = camera.position.x + camera.viewportWidth/2;
		float xRange = xMax - xMin;
		
		float yMin = camera.position.y - camera.viewportHeight/2;
		float yMax = camera.position.y + camera.viewportHeight/2;
		float yRange = yMax - yMin;
		
		Gdx.gl.glEnable(GL10.GL_BLEND);
		Gdx.gl.glBlendFunc(GL10.GL_SRC_ALPHA, GL10.GL_ONE_MINUS_SRC_ALPHA);

		shapeRenderer.setProjectionMatrix(camera.combined);
		shapeRenderer.begin(ShapeType.Line);
		shapeRenderer.setColor(1, 1, 1, 0.2f);
		
		float cellSize = 0.1f;
		
		for (float x = xMin; x <= xMax; x += cellSize) {
			shapeRenderer.line(x + cellSize/2, yMin, x + cellSize/2, yMax);
		}
		
		for (float y = yMin; y <= yMax; y += cellSize) {
			shapeRenderer.line(xMin, y + cellSize/2, xMax, y + cellSize/2);
		}
		
		shapeRenderer.end();

		Gdx.gl.glDisable(GL10.GL_BLEND);
	}

}
