package net.faintedge.radial.input;

import java.util.HashSet;
import java.util.Set;

import net.engio.mbassy.bus.MBassador;
import net.faintedge.radial.input.events.KeyDownEvent;
import net.faintedge.radial.input.events.KeyHeldEvent;
import net.faintedge.radial.input.events.KeyTypedEvent;
import net.faintedge.radial.input.events.KeyUpEvent;
import net.faintedge.radial.input.events.MouseMovedEvent;
import net.faintedge.radial.input.events.MouseScrolledEvent;
import net.faintedge.radial.input.events.TouchDownEvent;
import net.faintedge.radial.input.events.TouchDraggedEvent;
import net.faintedge.radial.input.events.TouchUpEvent;

import com.badlogic.gdx.InputProcessor;

public class InputManager implements InputProcessor {
	
	private Set<Integer> keysHeld = new HashSet<Integer>();
	private MBassador<Object> eventBus;
	
	public InputManager(MBassador<Object> eventBus) {
		this.eventBus = eventBus;
	}
	
	public void process(float tpf) {
		enqueueKeyHeldEvents(tpf);
	}
	
	private void enqueueKeyHeldEvents(float tpf) {
		for (Integer keycode : keysHeld) {
			eventBus.publish(new KeyHeldEvent(keycode, tpf));
		}
	}
	
	@Override
	public boolean keyDown(int keycode) {
		keysHeld.add(keycode);
		eventBus.publish(new KeyDownEvent(keycode));
		return false;
	}

	@Override
	public boolean keyUp(int keycode) {
		keysHeld.remove(keycode);
		eventBus.publish(new KeyUpEvent(keycode));
		return false;
	}

	@Override
	public boolean keyTyped(char character) {
		eventBus.publish(new KeyTypedEvent(character));
		return false;
	}

	@Override
	public boolean touchDown(int screenX, int screenY, int pointer, int button) {
		eventBus.publish(new TouchDownEvent(screenX, screenY, pointer, button));
		return false;
	}

	@Override
	public boolean touchUp(int screenX, int screenY, int pointer, int button) {
		eventBus.publish(new TouchUpEvent(screenX, screenY, pointer, button));
		return false;
	}

	@Override
	public boolean touchDragged(int screenX, int screenY, int pointer) {
		eventBus.publish(new TouchDraggedEvent(screenX, screenY, pointer));
		return false;
	}

	@Override
	public boolean mouseMoved(int screenX, int screenY) {
		eventBus.publish(new MouseMovedEvent(screenX, screenY));
		return false;
	}

	@Override
	public boolean scrolled(int amount) {
		eventBus.publish(new MouseScrolledEvent(amount));
		return false;
	}

}
